@@ -1,4 +1,4 @@
-plot.nmix<-function(x, which=1:5, separate=FALSE, plugin=FALSE, offset=1, nsamp=50, equi=TRUE, allsort=TRUE, ...) 
+plot.nmix<-function(x, which=1:5, separate=FALSE, plugin=FALSE, offset=1, nsamp=50, equi=TRUE, allsort=TRUE, trued=NULL, ...) 
 {
 opars<-par(no.readonly=TRUE)
 on.exit(par(opars))
@@ -10,7 +10,7 @@ switch(w,
 	{
 	layout(matrix(c(1, 2), 2, 1), heights = c(2, 1))
 	par(mar = c(4, 4, 2, 2))
-	pldens(x, add = TRUE, plugin)
+	pldens(x, plugin, trued)
 	np <- length(x$post) + min(0, 3 - match(FALSE, 0 == rev(x$post)))
 	barplot(x$post[1:np], names = 1:np)
 	title("posterior on k")
@@ -18,7 +18,7 @@ switch(w,
 ,
 	pltraces(x$traces,offset)
 ,
-	plsampden(x,nsamp,equi)
+	plsampden(x,nsamp,equi,,trued)
 ,
 	plcoin(x,allsort)
 ,
@@ -28,45 +28,53 @@ switch(w,
 invisible(NULL)
 }
 
-pldens<-function(x='fort.13',add=FALSE, plugin)
+pldens<-function(x, plugin, trued=NULL)
 {
-if(is.character(x)) 
+zd<-x$den
+if(!is.null(zd))
 	{
-	zd<-scan(x) 
-	zd<-matrix(zd,8,200,byrow=T)
-	} else {
-	if(is.list(x)) zd<-x$den else zd<-x
+	zdmax<-max(zd[-1,])
+	if(plugin&&!(is.null(x$pe)||is.null(x$post))) 
+		{
+		yg<-x$den[1,]
+		fyg<-rep(0,length(yg))
+		for(k in 1:length(x$pe)) if(x$post[k]>0)
+		   {
+		   w<-x$pe[[k]][,'wt']; m<-x$pe[[k]][,'mu']; s<-x$pe[[k]][,'sigma']
+		   fyg<-fyg+x$post[k]*apply(w*dnorm(outer(m,yg,"-")/s)/s,2,sum)
+		   }
+		zdmax<-max(zdmax,max(fyg))
+		}
+	if(!is.null(trued)) zdmax<-max(zdmax,max(trued(zd[1,])))
 	}
-if(add&!is.null(x$y)) 
+if(!is.null(x$y)) 
 	{
 	hist(x$y,50,prob=TRUE,main="",xlab="") 
-	if(is.null(zd)) title("histogram of data") else title("posterior expected density")
+	if(is.null(zd)) title("histogram of data") else 
+		{
+		if(zdmax>par('usr')[4]) hist(x$y,50,prob=TRUE,main="",xlab="",ylim=c(0,zdmax))
+		title("posterior expected density")
+		}
 	} else {
 	if(is.null(zd))
 		{
 		plot(0:1,0:1,type='n',axes=FALSE,xlab='',ylab='')
 		return(invisible(NULL))
 		}
-	plot(zd[1,],zd[2,],type='n',ylab='density',xlab='',ylim=c(0,max(zd[2:8,])))
+	plot(zd[1,],zd[2,],type='n',ylab='density',xlab='',ylim=c(0,zdmax))
 	title("posterior expected density")
 	}
 mtext(x$tag,1,line=2)
 for(j in 2:7) lines(zd[1,],zd[j,])
 lines(zd[1,],zd[8,],col='red',lwd=2)
+if(!is.null(trued)) lines(zd[1,],trued(zd[1,]),lwd=2,col='blue')
 if(!plugin) return()
 # add plug-in density estimator
 if(is.null(x$pe)||is.null(x$post)) return()
-yg<-x$den[1,]
-fyg<-rep(0,length(yg))
-for(k in 1:length(x$pe)) if(x$post[k]>0)
-{
-w<-x$pe[[k]][,'wt']; m<-x$pe[[k]][,'mu']; s<-x$pe[[k]][,'sigma']
-fyg<-fyg+x$post[k]*apply(w*dnorm(outer(m,yg,"-")/s)/s,2,sum)
-}
-lines(yg,fyg,type='l',col='blue',lwd=2,lty=2)
+lines(yg,fyg,type='l',col='darkgreen',lwd=2)
 }
 
-pltraces<-function(traces,offset=1,ccol=c('black','red','blue','darkgreen'))
+pltraces<-function(traces,offset=1,ccol=c('black','red','blue','blue'))
 {
 # only keep simple vectors, not matrices or lists
 traces<-traces[names(which(unlist(lapply(traces,function(x){is.vector(x)&!is.list(x)}))))]
@@ -93,11 +101,11 @@ mtext(names(traces)[itr],side,line=2,at=0.5+level,col=colr)
 }
 }
 
-plsampden<-function(z,nsamp=50,equi=TRUE,ngrid=200)
+plsampden<-function(z,nsamp=50,equi=TRUE,ngrid=200,trued=NULL)
 {
 if(is.null(z$traces$pars)) return()
 layout(1)
-par(mar=c(4,4,3,4)+.1)
+opars<-par(mar=c(4,4,3,4)+.1)
 
 yrange<-max(z$y)-min(z$y); yd0 = min(z$y)-0.05*yrange; ydinc = 1.1*yrange/ngrid
 yg<-yd0+ydinc*(1:ngrid)
@@ -108,11 +116,13 @@ dd<-matrix(0,ngrid,nsamp)
 for(it in seq_along(tw)) for(i in 1:ngrid)
 {
 t<-tw[it]
-dd[i,it]<-sum((dnorm((yg[i]-z$traces$pars[[t]][,2])/z$traces$pars[[t]][,3]))*z$traces$pars[[t]][,1])
+wms<-z$traces$pars[[t]]
+dd[i,it]<-sum(wms[,1]*dnorm(yg[i],wms[,2],wms[,3]))
 }
-matplot(yg,dd,type='l',lty=1,ylab='',xlab='')
+matplot(yg,dd,type='l',lty=1,ylab='',xlab='',col=c(1,2,3,5,6))
 abline(h=0)
 mtext("posterior sample of densities",side=3,line=1,cex=1.6)
+if(!is.null(trued)) lines(yg,trued(yg),lwd=2,col='blue')
 title(xlab=z$tag,line=2)
 title(ylab='density',line=2)
 }
@@ -141,12 +151,12 @@ if(missing(k))
 #layout(matrix(c(1,3,2,4),2,2))
 par(mfrow=c(2,2),mar=c(3,2,3,2)+.1)
 o<-order(z$post,decreasing=TRUE)
-ks<-sort((o[o<8])[1:4])
+ks<-sort((o[o<8&o>1])[1:4])
 } else {
 par(mfrow=c(1,1),mar=c(4,3,3,3)+.1)
 ks<-k
 }
-for(k in ks) 
+for(k in ks) if(k>1) 
 {
 ngrid<-200
 yrange<-max(z$y)-min(z$y); yd0 = min(z$y)-0.05*yrange; ydinc = 1.1*yrange/ngrid
